using System;
using System.Drawing;
using System.Drawing.Drawing2D;
using System.Drawing.Imaging;

namespace LunarDestroyer
{
	/// <summary>
	/// Implementacia naboja do hry Lunar Destroyer
	/// pricom naboj je dynamicky vykresleny.
	/// </summary>
	public class Naboj
	{
		public Point pPozicia;			//pozicia naboja
		private bool bAktivny = false;	//je aktivny
		private int iRychlost;			//rychlost
		private float fSmer;			//smer
		private bool bKolStop = false;	//zastavena kolizia?
		private int iMaxVrat = 4;		//max pocet obehov vo formulary
		private int iVrateni;			//pocet doposial obehov po formulary
		private Color Farba;			//farba naboja

		protected int iPriemer = 5;		//velkost vykresleneho naboja (5)

		//konstruktor
		public Naboj(int _startX, int _startY, int _rychlost, float _smer, Color _farba)
		{
			pPozicia = new Point();
			pPozicia.X = _startX;
			pPozicia.Y = _startY;
			Farba=_farba;

			iRychlost = _rychlost;
			fSmer = _smer;
			iVrateni = 0;
		}
		//vykresli naboj
		public void Kresli(Graphics _kresli)
		{
			if (!bAktivny)
				return;
			//vykresli na obrazovku
			_kresli.FillEllipse(new SolidBrush(Farba),pPozicia.X,pPozicia.Y,iPriemer,iPriemer);
		}
		//vlastnost rychlost
		public int Rychlost
		{
			get 
			{
				return iRychlost;
			}
			set
			{
				iRychlost = value;
			}
		}
		//stale v hre? - existuje po kolizii
		public bool Ostava
		{
			get 
			{
				return bKolStop;
			}
			set
			{
				bKolStop = value;
			}
		}
		//vlastnost aktivity naboja
		public bool Aktivny
		{
			get 
			{
				return bAktivny;
			}
			set
			{
				bAktivny = value;
			}
		}
		//pohyb naboja
		public void Pohyb()
		{
			//vypocita nove suradnice podla rychlosti
			int iNovaX = (int)(iRychlost * System.Math.Sin((System.Math.PI*fSmer)/180));
			int iNovaY = -(int)(iRychlost * System.Math.Cos((System.Math.PI*fSmer)/180));

			pPozicia.X = pPozicia.X + iNovaX;
			pPozicia.Y = pPozicia.Y + iNovaY;
		}
		//test na prechod vo vnutri nejakeho stvorca(formulara)
		public bool VnutriStvorca(Rectangle _stvorecTest)
		{
			//x laveho horneho rohu - je mimo dosah formulara
			if (pPozicia.X < _stvorecTest.X)		
			{
				if (iVrateni < iMaxVrat)	//a neprecerpal moznost este jedneho obehu
				{
					if (bKolStop)	//bol v kolizii
						pPozicia.X = _stvorecTest.X; // pohne na povodnu x
					else			//inak vykreslime do druheho okraja
						pPozicia.X = _stvorecTest.Right - -(pPozicia.X);
					iVrateni++;		//zvacsime pocet aktualnych vrateni
				}
				else return true;
			}
			//x praveho okraja
			if (pPozicia.X > _stvorecTest.Right)
			{
				if (iVrateni < iMaxVrat) 
				{
					if (bKolStop)
						//presunie na okraj
						pPozicia.X = _stvorecTest.Right - iPriemer; 
					else	//zobrazi na druhom okraji
						pPozicia.X = _stvorecTest.X + (_stvorecTest.Right - pPozicia.X); 
					iVrateni++;
				}
				else return true;
			}
			//y laveho horneho okraja
			if (pPozicia.Y < _stvorecTest.Y) 
			{
				if (iVrateni < iMaxVrat)
				{
					if (bKolStop)
						//pri kolizii necha na okraji
						pPozicia.Y = _stvorecTest.Y;
					else
						// inak zobrazi na druhy okraj
						pPozicia.Y = _stvorecTest.Bottom - -(pPozicia.Y);
					iVrateni++;
				}
				else return true;
			}
			//y dolneho okraja	
			if (pPozicia.Y > _stvorecTest.Bottom)
			{
				if (iVrateni < iMaxVrat)
				{
					if (bKolStop)
						//necha na dolnej pozicii
						pPozicia.Y = _stvorecTest.Bottom - iPriemer;
					else
						//vypocita novu hornu poziciu
						pPozicia.Y = _stvorecTest.Y + (_stvorecTest.Bottom - pPozicia.Y);
					iVrateni++;
				}
				else return true;
			}
			return false;
		}
		//vrati objekt stvorec symbolizujuci novu poziciu
		public Rectangle Okraje()
		{
			return new Rectangle(pPozicia.X, pPozicia.Y, iPriemer, iPriemer);
		}

		//osetri koliziu na urovni pixelov - naboj nema bitmapu
		public bool NarazilDo(Rectangle _stvorec, Bitmap _bitmap)
		{
			Rectangle rNaraza = new Rectangle(_stvorec.X,_stvorec.Y,_stvorec.Width,_stvorec.Height);

			Rectangle rTest = this.Okraje();
			
			// existuje prekrytie?
			if (!rNaraza.IntersectsWith(rTest))
				return false;
			// nastavi stvorec na zrazku
			rNaraza.Intersect(rTest);

			// GDI+ : vracajuci format je BGR, nie RGB.
			// osetruje sa iba bitmapa druhych objektov - naboj nema bitmapu
			BitmapData bmData2 = 
				_bitmap.LockBits(new Rectangle(rNaraza.X - _stvorec.X,rNaraza.Y - _stvorec.Y, rNaraza.Width,
				rNaraza.Height), ImageLockMode.ReadOnly, PixelFormat.Format24bppRgb);

			// velkost bitmapy v bitoch
			int stride = bmData2.Stride;
			// startovaci bod bitmapy
			System.IntPtr Scan1 = bmData2.Scan0;

			//praca s pametou
			unsafe
			{
				// pointer na byte
				byte * p1 = (byte *)Scan1;
				//vypocet poctu riadkov
				int nOffset = stride - rNaraza.Width*3;	
				for(int y=0;y<rNaraza.Height;++y)
				{
					for(int x=0; x < rNaraza.Width; ++x )
					{
						//ak je farba v zlozkach ina
						if (p1[0] != 0 && p1[1] != 0 && p1[2] != 0)
						{
							_bitmap.UnlockBits(bmData2);
							return true;
						}
						// pohni pointer na dalsi bit, kazdy pixel ma 3 byty
						p1 += 3;
					}
					// dalsi riadok
					p1 += nOffset;
				}
			}
			_bitmap.UnlockBits(bmData2);
			return false;			
		}
	}
}

