using System;
using System.Drawing;
using System.Drawing.Imaging;
using System.Drawing.Drawing2D;
using System.Collections;
using System.IO;

namespace LunarDestroyer
{
	/// <summary>
	/// Trieda pre vesmirnu lod v hre Lunar Lander.
	/// Obsahuje zakladne metody pre fungovanie vesmirnej lode.
	/// </summary>
	public class vesLod
	{
		public Point pPozicia;				//pozicia lode
		protected Rectangle stvorec;		//stvoruholnik velkosti obrazka
		protected double dRychlost = 0;		//rychlost lode
		protected float fRot = 0;			//uhol natocenia lode
		protected float fSmer = 0;			//smer aktualneho pohybu
		protected bool bAktivna;			//existuje lod = je Aktivny
		protected bool bAkceleruje;			//priznak akceleracie
		protected bool bNabojZotr = true;	//zotrvacnost naboja
		protected Color Farba;				//farba vystrelovanych striel

		// vlastnosti
		protected Bitmap imgLod = null;			//ukazovatel na obrazok
		protected Bitmap imgExplozia = null;	//to iste len explozie
		protected int iMaxRychl;				//max rychlost
		protected int iZrychl;					//zrychlenie
		protected int iHmota;					//hmotnost
		protected int iRychlOtac;				//rychlost otacania
		protected int iRychlStrely = 25;		//min. rychlost vystrelenej strely

		public Naboj[] Naboje;					//zasoba nabojov

		//konstruktor
		public vesLod(int _startX, int _startY, bool _zotrvacnost, float _otocenie, Color _farba)
		{ 
			pPozicia.X = _startX;
			pPozicia.Y = _startY;

			iMaxRychl = 20;
			iZrychl = 45;
			iHmota = 5;
			iRychlOtac = 15; 
			Naboje = new Naboj[10];
			bAktivna = true;
			bNabojZotr=_zotrvacnost;
			fRot = _otocenie;
			Farba=_farba;

			if (imgLod  == null)
			{
				imgLod = (Bitmap)Image.FromFile("Ship.gif");
			}
			imgLod.MakeTransparent(Color.Black);
		}
		//defaultny konstruktor - dedicnost robila problemy
		public vesLod()
		{
		}
		/*-----------Vlastnosti lode---------*/
		//vypocet stredu lode
		public Point Stred
		{
			get
			{
				return new Point(stvorec.X+stvorec.Width/2,stvorec.Y+stvorec.Height/2);
			}
		}
		public int Hmota
		{
			get
			{
				return iHmota;
			}
		}
		public bool NabojZotr
		{
			get
			{
				return bNabojZotr;
			}
			set
			{
				bNabojZotr = value;
			}
		}
		public double Rychlost
		{
			get 
			{
				return dRychlost;
			}
			set
			{
				dRychlost = value;
			}
		}

		public bool Aktivny
		{
			get 
			{
				return bAktivna;
			}
			set
			{
				bAktivna = value;
			}
		}
		public bool Akceleruje
		{
			get 
			{
				return bAkceleruje;
			}
		}
		public float Rotuje
		{
			get 
			{
				return fRot;
			}
		}

		public float Smer
		{
			get 
			{
				return fSmer;
			}
		}
		//vykresli lod
		public void Kresli(Graphics _kresli)
		{
			// stred stvoruholnika
			Point Pcenter = this.Stred;
			// pohne maticou tak aby 4-uholnik lode bol na 0,0
			_kresli.TranslateTransform(Pcenter.X, Pcenter.Y);
			//aplikuje rotaciu
			_kresli.RotateTransform(fRot);

			// umiestnime na povodnu poziciu
			stvorec.Offset(-Pcenter.X, -Pcenter.Y);

			// vykreslime obrazok na pov.pozicii
			_kresli.DrawImage(imgLod, stvorec,0,0,imgLod.Width,imgLod.Height, GraphicsUnit.Pixel);
			if (bAkceleruje) 
			{ 
				// vykreslime plamen
				_kresli.FillEllipse(new SolidBrush(Color.Yellow),stvorec.Left + stvorec.Width/2 - 3,stvorec.Bottom,5,14);
				_kresli.FillEllipse(new SolidBrush(Color.Red),stvorec.Left + stvorec.Width/2 - 2,stvorec.Bottom,3,6);
				bAkceleruje = false;
			}			
			// zmenime spat suradnice matice
			_kresli.ResetTransform();
		}
		//zrotuje dolava
		public void Dolava()
		{
			fRot -= iRychlOtac;
			if (fRot < 0)
				fRot += 360;	
		}
		//doprava
		public void Doprava()
		{
			fRot += iRychlOtac;
			if (fRot >= 360)
				fRot -= 360 ;
		}
		//akceleruje
		public void Akceleruj()
		{	
			//smer natocenia rozny od smeru akceleracie
			if (fSmer != fRot)
			{
				// vypocet dx,dy podla pomeru zrychlenia a hmotnosti -
				double dRotX = (iZrychl/iHmota * System.Math.Sin(System.Math.PI*fRot/180));  
				double dRotY = -(iZrychl/iHmota * System.Math.Cos(System.Math.PI*fRot/180));

				// podla rychlosti a smeru v ktorom chceme otacat
				double dDestX = (dRychlost * System.Math.Sin(System.Math.PI*fSmer/180));
				double dDestY = -(dRychlost * System.Math.Cos(System.Math.PI*fSmer/180));

				// spolu
				double dNovaX = dDestX + dRotX;
				double dNovaY = dDestY + dRotY;

				// nova rychlost
				dRychlost = System.Math.Sqrt((dNovaX * dNovaX) + (dNovaY * dNovaY));
				if (dRychlost > iMaxRychl)
					dRychlost = iMaxRychl;

				// novy smer pohybu
				if ((dNovaX == 0.0) || (dNovaY == 0.0))
					fSmer = fRot;
				else
				{
					//vypocet kvadrantu
					if (dNovaX > 0 && dNovaY > 0) 
						fSmer = (float)(System.Math.Atan(dNovaY/dNovaX)*(180/System.Math.PI) + 90);
					else if (dNovaX > 0)
						fSmer = (float)(System.Math.Atan(dNovaX/-dNovaY)*(180/System.Math.PI));
					else if (dNovaY > 0)
						fSmer = (float)(System.Math.Atan(-dNovaX/dNovaY)*(180/System.Math.PI) + 180);
					else
						fSmer = (float)(System.Math.Atan(-dNovaY/-dNovaX)*(180/System.Math.PI) + 270);
				}
				//* Debug
				//					TextWriter output = File.AppendText("debug.txt");
				//					output.WriteLine("dNovaY: "+dNovaY + @"|dNovaX: "+dNovaX + "|fdir: " + fSmer );
				//					output.Close();
				//* End				
			} 
			else 
			{
				//nahodnost pridania plynu - efekt nezastavitelnosti
				Random R = new Random();
				dRychlost += (iZrychl+R.Next(10))/iHmota;
				if (dRychlost > iMaxRychl)
					dRychlost = iMaxRychl;
			}
			// vykreslovanie zavoja za lodou
			bAkceleruje = true;

		}
		//vrati objekt stvorec reprezentujuci polohu lode
		public Rectangle Okraje()
		{
			return stvorec; // new Rectangle(pPozicia.X, pPozicia.Y, imgLod.Width, imgLod.Height);
		}
		//test ci sa nachadza lod vo formulari
		public bool VnutriStvorca(Rectangle _stvorecTest)
		{
			bool bHit = true;
			if (pPozicia.X < _stvorecTest.X)
			{
				pPozicia.X = _stvorecTest.Right - -(stvorec.X); 
				bHit = false;
			}
			if (pPozicia.X > _stvorecTest.Right)
			{
				pPozicia.X = _stvorecTest.X + (pPozicia.X - _stvorecTest.Right); 
				bHit = false;
			}
				
			if (pPozicia.Y < _stvorecTest.Y) 
			{
				pPozicia.Y = _stvorecTest.Bottom - -(stvorec.Y); 
				bHit = false;
			}
				
			if (pPozicia.Y > _stvorecTest.Bottom)
			{
				pPozicia.Y = _stvorecTest.Y + (pPozicia.Y - _stvorecTest.Bottom); 
				bHit = false;
			}
			return bHit;
		}
		

		//pohyb lode
		public void Pohyb()
		{
			// vypocet novych x,y na baze vektoru zrychlenia
			int iNovaX = (int)(dRychlost * System.Math.Sin(System.Math.PI*fSmer/180)); 
			int iNovaY = -(int)(dRychlost * System.Math.Cos(System.Math.PI*fSmer/180)); 

			pPozicia.X = pPozicia.X + iNovaX;
			pPozicia.Y = pPozicia.Y + iNovaY;
			//aktualizujeme poziciu
			stvorec = new Rectangle(pPozicia.X, pPozicia.Y, imgLod.Width, imgLod.Height);
		}
		
		//vystreli naboje
		public void Vystrel()
		{
			for(int j = 0;j<Naboje.Length;j++)
			{
				if((Naboje[j] == null) || (!Naboje[j].Aktivny))
				{
					if (bNabojZotr)
					{
						if (fSmer != fRot)
						{	
							float strel;
							// kade striela delo
							double dRotX = (iRychlStrely * Math.Sin(Math.PI*fRot/180));  
							double dRotY = -(iRychlStrely * Math.Cos(Math.PI*fRot/180));
							// kam strelu taha zotrvacnostou
							double dDestX = (dRychlost * Math.Sin(Math.PI*fSmer/180));
							double dDestY = -(dRychlost * Math.Cos(Math.PI*fSmer/180));
							// spolu
							double dNovaX = dDestX + dRotX;
							double dNovaY = dDestY + dRotY;

							// nova rychlost
							double Rychlost = Math.Sqrt((dNovaX * dNovaX) + (dNovaY * dNovaY));

							// novy smer pohybu
							if (dNovaX == 0.0)
								strel = 0;
							else
							{ 
								//vypocita uhol strely
								if (dNovaX > 0 && dNovaY > 0) 
									strel = (float)(Math.Atan(dNovaY/dNovaX)*(180/Math.PI) + 90);
								else if (dNovaX > 0)
									strel = (float)(Math.Atan(dNovaX/-dNovaY)*(180/Math.PI));
								else if (dNovaY > 0)
									strel = (float)(Math.Atan(-dNovaX/dNovaY)*(180/Math.PI) + 180);
								else
									strel = (float)(Math.Atan(-dNovaY/-dNovaX)*(180/Math.PI) + 270);
							}
							Naboje[j]= new Naboj(Predok().X,Predok().Y,(int)Rychlost,strel,Farba);
						} 
						else 
						{ 
							Naboje[j]= new Naboj(Predok().X,Predok().Y,(int)dRychlost+iRychlStrely,fRot,Farba);
						}
					} 
					else 
					{
						Naboje[j]= new Naboj(Predok().X,Predok().Y,(int)(dRychlost+iRychlStrely),fRot,Farba);
					}
					Naboje[j].Aktivny = true;
					j = Naboje.Length;
				}
			}
		}

		
		//vypocita suradnice anteny lode
		public Point Predok()
		{
			// polomer kruhu, ktory opisuje lod
			int iPolomer = (int)Math.Sqrt(Math.Pow(imgLod.Height/2,2) + Math.Pow(imgLod.Width/2,2));

			// prvy bod vpredu pred lodou
			Point pPredok = new Point();
			//vypocet novych suradnic (o 1 pixel viac ako je obrazok lode)
			pPredok.X = pPozicia.X + imgLod.Width/2 + (int)((iPolomer+1) * Math.Sin((Math.PI*fRot)/180));  
			pPredok.Y = pPozicia.Y + imgLod.Height/2 + -(int)((iPolomer+1) * Math.Cos((Math.PI*fRot)/180));

			return pPredok;
		}
		//kontrola kolizie na pixelovej urovni
		public bool NarazilDo(Rectangle _stvorec, Bitmap _bitmap)
		{
			Rectangle rNaraza = new Rectangle(_stvorec.X,_stvorec.Y,_stvorec.Width,_stvorec.Height);

			Rectangle rTest = this.Okraje();

			// kontrola narazu (ak nic,nepokracuje sa dalej
			if (!rNaraza.IntersectsWith(rTest))
				return false;
			
			// nastavi na zrazku
			rNaraza.Intersect(rTest);

			// GDI+ : vracajuci format je BGR, NOT RGB.
			// zamkneme prekryvajuce bity
			BitmapData bmData = imgLod.LockBits(new Rectangle(rNaraza.X - rTest.X,rNaraza.Y - rTest.Y, rNaraza.Width, rNaraza.Height),ImageLockMode.ReadOnly, PixelFormat.Format24bppRgb);
			BitmapData bmData2 = _bitmap.LockBits(new Rectangle(rNaraza.X - _stvorec.X,rNaraza.Y - _stvorec.Y, rNaraza.Width, rNaraza.Height),ImageLockMode.ReadOnly, PixelFormat.Format24bppRgb);

			// velkost bitmapy v bitoch
			int stride = bmData.Stride;
			// zaciatocny bod bitmapy
			IntPtr Scan0 = bmData.Scan0;
			IntPtr Scan1 = bmData2.Scan0;

			unsafe
			{
				// pointer na byte
				byte * p = (byte *)Scan0;
				byte * p1 = (byte *)Scan1;

				int nOffset = stride - rNaraza.Width*3;
	
				for(int y=0;y<rNaraza.Height;++y)
				{
					for(int x=0; x < rNaraza.Width; ++x )
					{
						if (p[0] != 0 &&  // je byte cierny?
							p[1] != 0 &&
							p[2] != 0 &&
							p1[0] != 0 &&
							p1[1] != 0 &&
							p1[2] != 0)
						{				  // ak nie nastala kolizia
							imgLod.UnlockBits(bmData);
							_bitmap.UnlockBits(bmData2);
							return true;
						}
						// presunieme pointer na novy pixel, kazdy ma 3 byte
						p += 3;
						p1 += 3;;
					}
					// presunieme na novy riadok
					p += nOffset;
					p1 += nOffset;
				}
			}

			imgLod.UnlockBits(bmData);
			_bitmap.UnlockBits(bmData2);

			return false;			
		}
		//vracia bitmapu lode
		public Bitmap LodImg
		{
			get
			{
				return imgLod;
			}
		}
		//virtualne metody
		public virtual bool Hrac()
		{
			return true;
		}
		public virtual void Prezi(ArrayList Asteroidy){}
	}

}

